import { Component } from "@angular/core";

import { FixedResults } from "../../results/fixed-results";
import { NgParameter } from "../../formulaire/ngparam";
import { CalculatorResults } from "../../results/calculator-results";
import { I18nService } from "../../services/internationalisation/internationalisation.service";
import { ApplicationSetupService } from "../../services/app-setup/app-setup.service";
import { FixedVarResultsComponent } from "./fixedvar-results.component";
import { ResultElement } from "jalhyd";

@Component({
    selector: "fixed-results",
    templateUrl: "./fixed-results.component.html",
    styles: [`
    .result_right {
        text-align: right;
    }
    .result_center {
        text-align: center;
    }
    .result_id_0 {
        background-color: #f0f0f0;
    }
    .result_id_1 {
        background-color: #ffffff;
    }
    .result_id_2 {
        font-weight: bolder;
        background-color: #00d0ff;
    }`]
})
export class FixedResultsComponent {
    /**
     * résultats non mis en forme
     */
    private _fixedResults: FixedResults;

    constructor(
        private intlService: I18nService,
        private appSetupService: ApplicationSetupService,
    ) { }

    public set results(r: FixedResults) {
        this._fixedResults = r;
    }

    private get hasParameterResult(): boolean {
        return this._fixedResults && this._fixedResults.hasResults;
    }

    private get resultElement(): ResultElement {
        return this._fixedResults && this._fixedResults.result && this._fixedResults.result.resultElement;
    }

    private get resultLabel(): string {
        return this._fixedResults && this._fixedResults.calculatedParameter && this._fixedResults.calculatedParameterHeader;
    }

    private get uitextParamFixes() {
        return this.intlService.localizeText("INFO_CALCULATOR_PARAMFIXES");
    }

    private get uitextValeurs() {
        return this.intlService.localizeText("INFO_CALCULATOR_VALEURS");
    }

    private getFixedParamClass(i: number) {
        // if (this._results.isFixed && i === this._results.fixedResults.length - 1)
        // return "font-weight-bold";
        // tslint:disable-next-line:no-bitwise
        return "result_id_" + String(i & 1);
    }

    private get fixedParams() {
        return this._fixedResults && this._fixedResults.fixedParameters;
    }

    public get hasFixedParameters() {
        return this._fixedResults && this._fixedResults.fixedParameters.length > 0;
    }

    private formattedLabel(p: NgParameter): string {
        return CalculatorResults.paramLabel(p, false);
    }

    private formattedValue(p: NgParameter): string {
        const nDigits = this.appSetupService.displayDigits;
        return p.getValue().toFixed(nDigits);
    }

    private get hasResults(): boolean {
        return this._fixedResults && this._fixedResults.hasResults;
    }
}
