import { Component, Input, AfterViewInit } from "@angular/core";
import { NgParameter } from "../../formulaire/ngparam";
import { DialogEditParamValuesComponent } from "../dialog-edit-param-values/dialog-edit-param-values.component";
import { MatDialog } from "@angular/material";
import { ParamValueMode } from "jalhyd";
import { I18nService } from "../../services/internationalisation/internationalisation.service";
import { sprintf } from "sprintf-js";

@Component({
    selector: "param-values",
    templateUrl: "./param-values.component.html",
    styleUrls: [
        "./param-values.component.scss"
    ]
})
export class ParamValuesComponent implements AfterViewInit {

    @Input()
    public param: NgParameter;

    @Input()
    public title: string;


    constructor(
        private editValuesDialog: MatDialog,
        private intlService: I18nService
    ) { }

    public get isMinMax() {
        return this.param.valueMode === ParamValueMode.MINMAX;
    }

    public get isListe() {
        return this.param.valueMode === ParamValueMode.LISTE;
    }

    public get infoText() {
        let text: string;
        if (this.isMinMax) {
            text = this.intlService.localizeText("INFO_PARAMFIELD_PARAMVARIER_MINMAXSTEP");
            text = sprintf(text, this.param.minValue, this.param.maxValue, this.param.stepValue);
        } else if (this.isListe) {
            text = this.intlService.localizeText("INFO_PARAMFIELD_PARAMVARIER_VALUES");
            const vals = this.param.valueList || [];
            text += vals.slice(0, 20).join(";");
        }
        return text;
    }

    public openDialog() {
        // modification de la valeur initiale, sans avoir à remettre le mode de
        // paramètre sur "fixé"
        this.editValuesDialog.open(
            DialogEditParamValuesComponent,
            {
                data: {
                    param: this.param
                }
            }
        );
    }

    public ngAfterViewInit() {
        // open dialog when switching to this mode, but only the first time this component is built,
        // otherwise switching back from another calc tab will trigger the dialog again
        if (this.param.minValue === undefined) {
            // use Promise trick to introduce a pseudo-timeout and avoid ExpressionChangedAfterItHasBeenCheckedError
            Promise.resolve().then(() => {
                this.openDialog();
            });
        }
    }
}
