import { Component } from "@angular/core";
import { Observable, IObservable, Observer } from "jalhyd";
import { GraphType } from "../../results/var-results";
import { I18nService } from "../../services/internationalisation/internationalisation.service";

@Component({
    selector: "graph-type",
    templateUrl: "../generic-select/generic-select.component.html"
})
export class GraphTypeSelectComponent implements IObservable {
    private _entries: GraphType[] = [GraphType.Histogram, GraphType.Scatter];
    private _entriesLabels: string[] = ["Histogramme", "XY"];
    private _selected: GraphType;

    private _observable: Observable;

    constructor(private intlService: I18nService) {
        this._observable = new Observable();
    }

    public get entries(): GraphType[] {
        return this._entries;
    }

    protected entryLabel(entry: GraphType): string {
        const i = this._entries.indexOf(entry);
        return this._entriesLabels[i];
    }

    public get selectedValue(): GraphType {
        return this._selected;
    }

    public set selectedValue(v: GraphType) {
        if (this._selected !== v) {
            this._selected = v;
            this.notifyObservers({
                "action": "select",
                "value": v
            }, this);
        }
    }

    public get label() {
        return this.intlService.localizeText("INFO_PARAMFIELD_GRAPH_TYPE");
    }

    // interface IObservable

    /**
     * ajoute un observateur à la liste
     */
    addObserver(o: Observer) {
        this._observable.addObserver(o);
    }

    /**
     * supprime un observateur de la liste
     */
    removeObserver(o: Observer) {
        this._observable.removeObserver(o);
    }

    /**
     * notifie un événement aux observateurs
     */
    notifyObservers(data: any, sender?: any) {
        this._observable.notifyObservers(data, sender);
    }
}
