import { NG_VALIDATORS, Validator, AbstractControl, ValidatorFn } from "@angular/forms";
import { Directive, Input } from "@angular/core";
import { NgBaseParam } from "../components/base-param-input/base-param-input.component";

/**
 * Synchronous validator for Ngparam, relying on JaLHyd ParamDefinition model
 */
@Directive({
    selector: "[appJalhydModelValidation]",
    providers: [ {
        provide: NG_VALIDATORS,
        useExisting: JalhydModelValidationDirective,
        multi: true
    } ]
})
export class JalhydModelValidationDirective implements Validator {
    @Input("appJalhydModelValidation") ngBaseParam: NgBaseParam;

    validate(control: AbstractControl): { [key: string]: any } | null {
        const mv = jalhydModelValidator(this.ngBaseParam)(control);
        return mv;
    }
}

export function jalhydModelValidator(ngBaseParam: NgBaseParam): ValidatorFn {
    return (control: AbstractControl): { [key: string]: any } | null => {
        let errorReturn = null; // no error, everything OK
        const result = ngBaseParam.validateModelValue(control.value);
        if (result && ! result.isValid) {
            errorReturn = {
                "jalhydModel": {
                    value: control.value,
                    message: result.message
                }
            };
        }
        return errorReturn;
    };
}
