import { Structure, Nub, ParallelStructure, StructureType, LoiDebit, StructureProperties, Props } from "jalhyd";

import { FormulaireDefinition } from "../form-definition";
import { CalculatorResults } from "../../../results/calculator-results";
import { FormDefParamToCalculate } from "../form-def-paramcalc";
import { FormDefFixedVar } from "../form-def-fixedvar";
import { FormDefParallelStructures } from "../form-def-parallel-structures";
import { FormComputeParallelStructures } from "../form-compute-parallel-structures";
import { FormResultFixedVar } from "../form-result-fixedvar";
import { FieldsetContainer } from "../../fieldset-container";
import { FieldSet } from "../../fieldset";
import { SelectField } from "../../select-field";
import { NgParameter } from "../../ngparam";
import { FieldsetTemplate } from "../../fieldset-template";
import { FormulaireNode } from "../../formulaire-node";

export class FormulaireParallelStructure extends FormulaireDefinition {

    // private _formFixedVar: FormDefFixedVar;

    private _formParallelStruct: FormDefParallelStructures;

    private _formParamCalc: FormDefParamToCalculate;

    private _formCompute: FormComputeParallelStructures;

    private _formResult: FormResultFixedVar;

    /**
     * id du select configurant le type d'ouvrage
     */
    private __ouvrageSelectId: string;

    constructor() {
        super();
        // this._formFixedVar = new FormDefFixedVar(this);
        this._formParamCalc = new FormDefParamToCalculate(this);
        this._formResult = new FormResultFixedVar(this, false);
        this._formParallelStruct = new FormDefParallelStructures();
        this._formCompute = new FormComputeParallelStructures(this, this._formParallelStruct, this._formResult);
    }

    private createStructNub(templ: FieldsetTemplate): Nub {
        // !!! attention !!!
        // Il doit y avoir cohérence dans le fichier de conf entre les valeurs defaultXXX et les valeurs possibles pour les select
        // cad valeur par défaut du 1er select (type d'ouvrage), du 2ème (loi de débit).
        // A terme, il faudrait analyser le fichier de conf (dépendances d'existence) pour déterminer automatiquement ces valeurs

        const params = {};
        params["calcType"] = templ.calcTypeFromConfig;
        params["nodeType"] = templ.defaultNodeTypeFromConfig;
        params["structureType"] = templ.defaultStructTypeFromConfig;
        params["loiDebit"] = templ.defaultLoiDebitFromConfig;

        return this.createNub(params);
    }

    /**
     * ajoute un Nub Structure
     * @param st structure à ajouter
     * @param after position après laquelle insérer la structure, à la fin sinon
     */
    private addStructureNub(st: Structure, after?: number) {
        this.parallelStructureNub.addStructure(st, after);
    }

    private get parallelStructureNub(): ParallelStructure {
        return this.currentNub as ParallelStructure;
    }

    public createFieldset(parent: FormulaireNode, json: {}, data?: {}): FieldSet {
        if (json["calcType"] === "Structure") {
            // indice après lequel insérer le nouveau FieldSet
            const after = data["after"];

            const res: FieldSet = new FieldSet(parent);
            const sn = this.createStructNub(data["template"]);
            this.addStructureNub(sn as Structure, after);
            res.setNub(sn, false);

            if (after !== undefined) {
                parent.kids.splice(after + 1, 0, res);
            } else {
                parent.kids.push(res);
            }

            this.resetResults();

            return res;
        } else {
            return super.createFieldset(parent, json, data);
        }
    }

    protected initParse() {
        this._formParamCalc.initParse();
    }

    protected parseOptions(json: {}) {
        super.parseOptions(json);

        // id du select configurant le type d'ouvrage
        this.__ouvrageSelectId = this.getOption(json, "ouvrageSelectId");
    }

    public afterParseFieldset(fs: FieldSet) {
        // si le FieldSet contient le select de type d'ouvrage
        if (this.__ouvrageSelectId) {
            const sel = fs.getFormulaireNodeById(this.__ouvrageSelectId);
            if (sel) {
                // on abonne le formulaire aux propriétés du FieldSet
                fs.properties.addObserver(this);
            }
        }
    }

    public moveFieldsetUp(fs: FieldSet) {
        if (fs.nub instanceof Structure) {
            // déplacement du nub
            fs.nub.parent.moveStructureUp(fs.nub);
            // déplacement du fieldset
            this.fieldsetContainer.moveFieldsetUp(fs);

            this.resetResults();
        } else {
            super.moveFieldsetUp(fs);
        }
    }

    public moveFieldsetDown(fs: FieldSet) {
        if (fs.nub instanceof Structure) {
            // déplacement du nub
            fs.nub.parent.moveStructureDown(fs.nub);
            // déplacement du fieldset
            this.fieldsetContainer.moveFieldsetDown(fs);

            this.resetResults();
        } else { super.moveFieldsetDown(fs); }
    }

    public removeFieldset(fs: FieldSet) {
        if (fs.nub instanceof Structure) {
            // suppression du nub
            this._paramService.deleteNub(fs.nub);

            // suppression du fieldset
            this.fieldsetContainer.removeFieldset(fs);

            this.resetResults();
        } else { super.removeFieldset(fs); }
    }

    protected completeParse(json: {}) {
        this._formParamCalc.parseOptions(json);
        this.subscribeFieldsetContainer();
    }

    /**
     * gestion du clic sur les radios "paramètre fixé, à varier, à calculer"
     */
    public onRadioClick(info: string) {
        super.onRadioClick(info);
        this._formParamCalc.onRadioClick(info);
    }

    /**
     * @return une chaîne représentant le "contexte" courant (ici, combinaison type d'ouvrage-loi de débit)
     * @param fs FieldSet contenant les listes déroulantes type d'ouvrage et loi de débit
     */

    public resetResults() {
        this._formResult.resetResults();
    }

    public doCompute() {
        this._formCompute.doCompute();
    }

    public get hasResults(): boolean {
        return this._formResult.hasResults;
    }

    public get results(): CalculatorResults[] {
        return this._formResult.results;
    }

    private get fieldsetContainer(): FieldsetContainer {
        const n = this.getFormulaireNodeById("struct_container");
        if (n === undefined || !(n instanceof FieldsetContainer)) {
            throw new Error("l'élément 'struct_container' n'est pas du type FieldsetContainer");
        }
        return n as FieldsetContainer;
    }

    /**
     * recherche dans une config le 1er select avec une entrée "enum" avec une valeur donnée
     */
    private findSelectWithEnum(conf: {}, enumName: string, enumVal: string): {
        "select": { [key: string]: string; },
        "entry": { [key: string]: string; }
    } {
        let select; // select
        let selectEntry; // entrée du select correspondant à la valeur de l'enum

        for (const k in conf) {
            const kid = conf[k];
            if (kid.type === "select") {
                // entrées
                for (const e of kid.select) {
                    if (e.enum !== undefined) {
                        const tmp = e.enum.split(".");
                        const found = tmp[0] === enumName && tmp[1] === enumVal;
                        if (found) {
                            select = kid;
                            selectEntry = e;
                            break;
                        }
                    }
                }
                if (select !== undefined) {
                    break;
                }
            }
        }

        if (select !== undefined) {
            return { "select": select, "entry": selectEntry };
        }
    }

    /**
     * recherche dans une config le 1er objet avec dépendance donnée
     */
    private findObjectWithDependency(conf: {}, refid: string, refvalue: string): {} {
        for (const k in conf) {
            const kid = conf[k];
            if (kid.dep_exist !== undefined) {
                for (const dep of kid.dep_exist) {
                    if (dep.refid === refid && dep.refvalue === refvalue) {
                        return kid;
                    }
                }
            }
        }
    }

    /**
     * trouve une loi de débit compatible avec un type de structure dans un Fieldset en tenant compte des dépendances entre selects
     * @param fs
     * @param structType
     */
    private adjustLoiDebit(fs: FieldSet, structType: StructureType): Props {
        const res: Props = fs.properties.clone();

        // recherche du select "type de structure"

        const structSelect = this.findSelectWithEnum(fs.jsonConfig["fields"], "StructureType", StructureType[structType]);
        if (structSelect === undefined) {
            throw new Error(`pas de select trouvé pour la propriété StructureType=${StructureType[structType]}`);
        }

        // recherche du select dépendant (lois de débit)

        const loidebitSelect = this.findObjectWithDependency(fs.jsonConfig["fields"], structSelect.select.id, structSelect.entry.id);
        if (loidebitSelect === undefined || loidebitSelect["type"] !== "select") {
            // tslint:disable-next-line:max-line-length
            throw new Error(`pas de select trouvé avec une dépendance au select 'type de structure' pour la valeur ${structSelect.select.id}=${structSelect.entry.id} (1)`);
        }

        // liste des enums du select dépendant

        const loisDebit = [];
        for (const e of loidebitSelect["select"]) {
            if (e.enum !== undefined) {
                const tmp = e.enum.split(".");
                loisDebit.push(LoiDebit[tmp[1]]);
            }
        }

        if (loisDebit.length === 0) {
            // tslint:disable-next-line:max-line-length
            throw new Error(`pas de select trouvé avec une dépendance au select 'type de structure' pour la valeur ${structSelect.select.id}=${structSelect.entry.id} (2)`);
        }

        res.setPropValue("loiDebit", StructureProperties.findCompatibleLoiDebit(structType, loisDebit));

        return res;
    }

    // private adjustTypeStruct(fs: FieldSet, loiDebit: LoiDebit): Props {
    //     return undefined;
    // }

    /**
     * après une modification, détermine si les propriétés d'un Fieldset sont compatibles entre elles et les ajuste au besoin
     * @param props propriétés à vérifier
     * @param name nom de la propriété qui vient de changer
     * @param val nouvelle valeur de la propriété
     */
    private adjustProperties(fs: FieldSet, name: string, val: any): Props {
        // si prop=type d'ouvrage, on prend la 1ère loi de débit compatible avec
        // (spécifique aux ouvrages //), en tenant compte des dépendances.
        // (par ex, s'il existe un select de lois de débit dépendant du select de types
        // d'ouvrage, on prend la 1ère entrée du select de lois de débit compatible)
        if (name === "structureType") {
            if (!StructureProperties.isCompatibleValues(val, fs.properties.getPropValue("loiDebit"))) {
                return this.adjustLoiDebit(fs, val);
            }
        }
        // si prop=loi débit, on prend un type d'ouvrage compatible
        // else if (name === "loiDebit")
        //     if (!StructureProperties.isCompatibleValues(fs.properties.getPropValue("structureType"), val))
        //         return this.adjustTypeStruct(fs, val);

        return fs.properties;
    }

    /**
     * abonnement en tant qu'observateur du FieldsetContainer
     */
    private subscribeFieldsetContainer() {
        this.fieldsetContainer.addObserver(this);
    }

    /**
     * abonnement en tant qu'observateur des NgParameter des FieldSet contenus dans le FieldsetContainer
     */
    private subscribeStructureInputFields(fs: FieldSet) {
        for (const n of fs.allFormElements) {
            if (n instanceof NgParameter) {
                n.addObserver(this);
            }
        }
    }
    /**
     * abonnement en tant qu'observateur du SelectField des FieldSet contenus dans le FieldsetContainer
     */
    private subscribeStructureSelectFields(fs: FieldSet) {
        for (const n of fs.allFormElements) {
            if (n instanceof SelectField) {
                n.addObserver(this);
            }
        }
    }

    // interface Observer

    public update(sender: any, data: any) {
        if (sender instanceof FieldsetContainer) {
            switch (data.action) {
                case "newFieldset":
                    this.reset();
                    this.subscribeStructureInputFields(data["fieldset"]);
                    this.subscribeStructureSelectFields(data["fieldset"]);
            }
        } else if (sender instanceof FieldSet && data.action === "propertyChange") {
            switch (sender.id) {
                case "fs_ouvrage":
                    const props = this.adjustProperties(sender, data["name"], data["value"]);
                    const newNub = this.replaceNub(sender.nub, props);
                    sender.setNub(newNub);
                    this.reset();
                    break;
            }
        }
    }
}
